/* Program: importOBJ
 *
 * Mirela Damian
 * March 19 2013
 * Adapted from Nate Robin's lightposition.c tutor
 *
 * Import a scene of objects created with Blender, and saved in wavefront .obj format.
 * Important: .obj and .mtl files must be in the Debug space of the project directory.
 *            If not, the entire path to the .obj file must be supplied to glmReadOBJ.
 * Hitting space rotates the scene about an axis selected by pressing the keys x, y, z.
 */

#ifdef __APPLE__
#include <unistd.h>
#include <GLUT/glut.h>
#else
#include <windows.h>
#include <GL/glut.h>
#endif

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "glm.h"

#define WinW    500
#define WinH    500
#define UserW   3

enum axis {X=0, Y, Z};

/* Globals (axis, angle) used for scene rotation */
static GLint axis = Z;
/* Scene rotation angles to alter interactively */
static GLfloat theta[] = {0.0,0.0,0.0};

/* Pointer to model imported from Blender */
GLMmodel* pmodel = NULL;

/* Spin scene 2 degrees about selected axis */
void spinScene()
{
	theta[axis] += 2.0;
	if( theta[axis] > 360.0 ) theta[axis] -= 360.0;
	
	glutPostRedisplay();
}

/*
 * A keyboard event occurs when the user presses a key:
 * ' ' should cause the scene to rotate
 * x, y, z should select the rotation axis
 */
void Keyboard(unsigned char key, int x, int y)
{
	switch (key)
	{
        case ' ':   spinScene();
            break;
        case 'x':	axis = X;
            break;
        case 'y':	axis = Y;
            break;
        case 'z':	axis = Z;
            break;
        case 'q':	exit(0);
	}
	glutPostRedisplay();
}

/*
 *  This function is called whenever the display needs to redrawn.
 *  First call when program starts.
 */
void Display( void )
{
	/* Draw to the back buffer */
	glDrawBuffer( GL_BACK );
	
	/* Clear the display */
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    
	/* Alter the MODELVIEW matrix */
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glRotatef(theta[0], 1.0, 0.0, 0.0);
	glRotatef(theta[1], 0.0, 1.0, 0.0);
	glRotatef(theta[2], 0.0, 0.0, 1.0);
    
	/* Insert graphics code here that draws the scene */
	drawAxes();
    
	/* Draw entire model at once */
    glmDraw(pmodel, GLM_SMOOTH | GLM_MATERIAL); /* draw using colors or material */
    
	/* Draw model broup by group if needed */
	/* glmDrawGroup(pmodel, " Cube1", GLM_SMOOTH | GLM_MATERIAL);
	 * glTranslatef(0, 1, 0);
     * glmDrawGroup(pmodel, " Cube2", GLM_SMOOTH | GLM_MATERIAL);
     */
    
	/* Before returning, flush the graphics buffer so all graphics appear in the window */
	glFlush();
	glutSwapBuffers();
}

/*
 * Read a model from a wavefront .obj file. Scale it to a unit box.
 * Compute vertex and facet normals for lighting calculations.
 */
GLMmodel* readModel(char * filename)
{
	GLMmodel * model;
	
    model = glmReadOBJ(filename);
	if (!model) { printf("Ooops! Couldn't read model file. \n"); fflush(stdout); exit(0);}
	glmUnitize(model);
    glmFacetNormals(model);
    glmVertexNormals(model, 90.0);
	return model;
}

/*
 * Set window attributes
 */
void myInit()
{
	/* Set color used when clearing the window to black */
	glClearColor( 0.0, 0.0, 0.0, 0.0 );
    
	/* Set drawing color to white */
	glColor3f( 1.0, 1.0, 1.0 );
    
	/* Set up orthographic projection */
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glOrtho( -UserW, UserW, -UserW, UserW, -2.0, 2.0 );
    
    glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
    
	/* Enable hidden--surface--removal */
	glEnable(GL_DEPTH_TEST);
    
    /* Enable lighting */
	glEnable(GL_LIGHTING);
    glEnable(GL_NORMALIZE);
	glEnable(GL_LIGHT0);
}

int main( int argc, char ** argv )
{
	/* Initialize graphics window */
	glutInit( &argc, argv );
    
	/* Need both double buffering and z buffer */
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    
	/* Set size of graphics window and position of upper left corner on display */
	glutInitWindowSize(WinW, WinH );
	glutInitWindowPosition( 0, 0 );
    
	/* Create the window */
	glutCreateWindow( "Importing objects from Blender" );
    
	/* Register callback functions */
	glutDisplayFunc( Display );
    glutKeyboardFunc( Keyboard );
    
	/* Set window attributes */
	myInit();
    
	/* Read the wavefront .obj file exported from Blender.	*/
    /* Note: dolphins.obj and dolphin.mtl come with Nate Robin's tutors. */
    /* These two files have been copied in the Debug directory of this project. */
	pmodel = readModel("dolphins.obj");
    
	/* Start event processing */
	glutMainLoop();
}
